#include "../../proj/tl_common.h"
#include "../../proj/mcu/watchdog_i.h"
#include "../../proj_lib/rf_drv.h"
#include "../../proj_lib/pm.h"
#include "../common/rf_frame.h"
#include "../link_layer/rf_ll.h"
#include "dongle_custom.h"
#include "dongle_usb.h"
#include "dongle_emi.h"
#include "dongle_suspend.h"

extern u8 rf_tx_flag;
extern callback_rx_func p_post ;
extern  u8 *raw_pkt;


#define USB_FIFO_NUM   8
#define USB_FIFO_MAX_LEN  10
typedef struct
{
	u8 fifo[USB_FIFO_NUM][USB_FIFO_MAX_LEN];
	u8 wptr;
	u8 rptr;
}USB_FIFO_DATA_S;

typedef struct
{
	u8 type;
	u8 buf[USB_FIFO_MAX_LEN-1];
	
}USB_DATA_S;


#define RF_DATA_FIFO_NUM  4 
#define RF_DATA_FIFO_MAX_LEN  35
typedef struct
{
	u8 fifo[RF_DATA_FIFO_NUM][RF_DATA_FIFO_MAX_LEN];
	u8 wptr;
	u8 rptr;
}RF_DATA_FIFO_DATA_S;

typedef struct
{
	u8 type;
	u8 pno;
	u8 key_index;
	ENC_RF_DATA dat;
	
}RF_UI_DATA_S;



typedef enum
{
   NORMAL_KB_DATA_TYPE=1,
   MOUSE_DATA_TYPE=2,
   CONSUME_DATA_TYPE=3,
   SYSTEM_DATA_TYPE=4,  
}USB_DATA_TYPE_ENUM;

USB_FIFO_DATA_S usb_fifo_aaa;
RF_DATA_FIFO_DATA_S rf_data_fifo;
u32 kb_tick=0;
u32 ms_tick=0;
u8 has_consume_key_press=0;
u8 has_normal_key_press=0;
u8 has_system_key_press=0;

u8 mouse_not_release=0;

const u16 consumer_list[]={
0x221,		//0xa3 C_WWW_SEARCH		
0x223,		//0xa4 C_WWW_HOME
0x224,		//0xa5 C_WWW_BACKWARD
0x225,		//0xa6 C_WWW_FORWARD
0x226,		//0xa7 C_WWW_STOP
0x227,		//0xa8 C_WWW_REFRESH
0x22A,		//0xa9 C_MY_FAVORITE
0x183,		//0xaa C_MEDIA_SELECT
	
0x18A,		//0xab C_EMAIL
0x192,		//0xac C_CALCULATOR
0x194,		//0xad C_MY_COMPUTER
0xB5,		//0xae C_NEXT_TRACK
0xB6,		//0xaf C_PRE_TRACK
0xB7,		//0xb0 C_STOP
0xCD,		//0xb1 C_PLAY_PAUSE
0xE2,		//0xb2 C_MUTE	
	
0xE9,		//0xb3 C_VOL_INC
0xEA,		//0xb4 C_VOL_DEC	
0x00,		//0xb5 telinkԶ
0x22D,		//0xb6 USAGE ZOOM IN
0x22E,		//0xb7	 USAGE ZOOM OUT 
0x236,		//0xb8	 USAGE PAN LEFT
0x237,		//0xb9	 USAGE PAN RIGHT
0x30B,		//0xba	C_BRIGHT_INC	
	
0x30A,		//0xbb	C_BRIGHT_DEC
0xB8,		//0Xbc	 c_rject
0x30,		//0Xbd	C_POWER 		
0x19E,		//0Xbe	C_TERMINAL_LOCK 
};



u8 push_usb_fifo_aaa(u8 type,u8 *buf,u8 len)
{
	USB_DATA_S *p=(USB_DATA_S*)usb_fifo_aaa.fifo[usb_fifo_aaa.wptr&(USB_FIFO_NUM-1)];
	p->type=type;
	memcpy(p->buf,buf,len);
	usb_fifo_aaa.wptr++;
	int fifo_use=(usb_fifo_aaa.wptr-usb_fifo_aaa.rptr)&(USB_FIFO_NUM*2-1);
	if(fifo_use>USB_FIFO_NUM)
	{
		usb_fifo_aaa.rptr++;//overlap older data
	}
	return 1;
}
u8 push_rf_data_fifo(u8 type,u8 pno,u8 key_index,u8 *buf,u8 len)
{
	RF_UI_DATA_S *p=(RF_UI_DATA_S*)rf_data_fifo.fifo[rf_data_fifo.wptr&(RF_DATA_FIFO_NUM-1)];
	p->type=type;
	p->pno=pno;
	p->key_index=key_index;
	u8 *p_src=(u8*)&p->dat;
	memcpy(p_src,buf,len);
	rf_data_fifo.wptr++;
	int fifo_use=(rf_data_fifo.wptr-rf_data_fifo.rptr)&(RF_DATA_FIFO_NUM*2-1);
	if(fifo_use>RF_DATA_FIFO_NUM)
	{
		rf_data_fifo.rptr++;//overlap older data
	}
	return 1;
}


rf_packet_ack_t rf_ack_buf=
{
	0,	//dma_len

    0,	//rf_len
    RF_PROTO_BYTE,	// 0x51,proto
    PKT_FLOW_DIR,	// 0x80,flow
    FRAME_TYPE_ACK,	// 0x80,type
    
   //buf
};
void set_ack_buf_head(u8 packet_len,u8 type)
{
	user_rf_start_tx();
	rf_ack_buf.dma_len=packet_len-4;
	rf_ack_buf.rf_len=packet_len-5;
	rf_ack_buf.type=type;
}

_attribute_ram_code_ void irq_handler(void)
{
#if (FREQ_HOPPING_METHOD==1)

    u32 src = reg_irq_src;

    if (src & FLD_IRQ_TMR1_EN)
    {
        irq_host_timer1();
        reg_tmr_sta = FLD_TMR_STA_TMR1;//write 1 to clear
    }
#endif

    u16  src_rf = reg_rf_irq_status;
    if (src_rf & FLD_RF_IRQ_RX)
    {

        irq_host_rx();
        //log_event (TR_T_irq_rx);
    }

    if (src_rf & FLD_RF_IRQ_TX)
    {

        irq_host_tx();
    }
}

void check_first_normal_data()
{
#if (GOLD_TEST_DONGLE==0)	
	static u32 first_rx_normal_data_tick=0;
	if(first_rx_normal_data_tick==0)
	{
		first_rx_normal_data_tick=clock_time()|1;
	}
	else if(rf_pair_enable&&clock_time_exceed(first_rx_normal_data_tick,1000000))
	{
		clear_pair_flag();
	}
#endif
}
#if 0

void	callback_pairing(u8 *p)
{
    rf_packet_pairing_t *p_pkt = (rf_packet_pairing_t *)(p + 8);

    static u32 rx_rssi = 0;
   // if ((p_pkt->flow & PKT_FLOW_PARING))
    {
        static u8 rssi_last;
        if (!rx_rssi)
        {
            rx_rssi = RECV_PKT_RSSI(p);
        }
        u8 rssi_cur = RECV_PKT_RSSI(p);
        if (abs(rssi_last - rssi_cur) < 17)
        {
            rx_rssi = ((rx_rssi << 1) + rssi_cur) / 3   ;
        }
        rssi_last = rssi_cur;
    }
    //    rx_rssi = RECV_PKT_RSSI(p);
  
    int  rssi_paring_good       = rx_rssi > (23 - custom_rssi_paring_th_offset); 

    if ( (rssi_paring_good))
    {
        set_device_id_in_firmware(p_pkt);
        
    }

}
#endif
u8 dec_result[32]={0,0};
//ENC_RF_DATA dec;

void	callback_mouse(u8 *p)
{
    rf_packet_mouse_t *p_pkt = (rf_packet_mouse_t *)(p + 8);

	u8 new_data_num = 0; 
	mouse_data_t *now_ms_data;
	static u8 last_ms_btn;

    static u8 seq_no_mouse = 0;
   new_data_num = p_pkt->pno&0x0f;
   
    ms_tick=clock_time()|1;
	user_resume_host();
    if (p_pkt->seq_no != seq_no_mouse)
    {
        seq_no_mouse = p_pkt->seq_no;
		if ((reg_usb_host_conn==0)||(new_data_num==0))
		{
			return;
		}
       
		return;

#if (AES_METHOD ==0)
		u8 *p_dec=(u8*)&p_pkt->dat;
#elif((AES_METHOD ==2)||(AES_METHOD ==3))
		u8 *p_src=(u8*)&p_pkt->dat;
	#if DEBUG_GPIO_AAA
			gpio_write(PIN_DEBUG_AES,1);
	#endif	
	set_aes_key(p_pkt->type);
	aes_user_ecb_encryption(&p_src[0],&dec_result[0]);
	if(new_data_num>1)
	{
		aes_user_ecb_encryption(&p_src[16],&dec_result[16]);
	}
	#if DEBUG_GPIO_AAA
			gpio_write(PIN_DEBUG_AES,0);
	#endif	
	u8 *p_dec=(u8*)&dec_result[8];
	#if 0
		static u32 seq_no = 0;
		
		//if ((seq_no < dec.seq_no) || (seq_no == 0) || (0 == dec.seq_no))
		if ((seq_no != dec.seq_no))
		{
			seq_no = dec.seq_no;
		}
		else
		{
			return ;
		}
	#endif
#endif

		for(u8 i=0;i<new_data_num;i++)
		{
			//static u16 sn=0xffff;
			
			//if(sn!=p_dec[i*sizeof(mouse_data_t)+sizeof(mouse_data_t)-1])
			{
				//sn =p_dec[i*sizeof(mouse_data_t)+sizeof(mouse_data_t)-1];
				now_ms_data=(mouse_data_t *)&p_dec[i*sizeof(mouse_data_t)];
				
				if(now_ms_data->x||now_ms_data->y||now_ms_data->wheel||(last_ms_btn!=now_ms_data->btn))
				{
					last_ms_btn=now_ms_data->btn;
					push_usb_fifo_aaa(MOUSE_DATA_TYPE,&p_dec[i*sizeof(mouse_data_t)],6);
				}
				
			}
		}
	}

       

}

u8 kb_data_handle(u8 *buf)
{
	
	static u16 last_consume_key=0;
	static u8 last_system_key=0;
	static u8 last_normal_key[8]={0,0,0,0,0,0,0,0};
	
	u16 consume_key=0;
	u8 system_key=0;
	u8 normal_key[8]={0,0,0,0,0,0,0,0};
	u8 cnt=0;
	kb_data_t *kb=(kb_data_t *)buf;
	normal_key[0]=kb->ctrl_key;

     if(kb->cnt)
     {
		cnt=2;
		for(u8 i=0;i<kb->cnt;i++)
		{
			if((kb->keycode[i]>=0xa0)&&(kb->keycode[i]<=0xa2))
			{
				system_key=1<<(kb->keycode[i]-0xa0);
				has_system_key_press=1;
			}
			else if((kb->keycode[i]>=0xa3)&&(kb->keycode[i]<=0xbe))
			{
				consume_key=consumer_list[kb->keycode[i]-0xa3];
				has_consume_key_press=1;
			}
			else
			{
				normal_key[cnt]=kb->keycode[i];
				cnt++;
				has_normal_key_press=1;
			}
		}
     }
	if(normal_key[0])
	{
		has_normal_key_press=1;
	}
	if(memcmp(last_normal_key,normal_key,8))
	{
		memcpy(last_normal_key,normal_key,8);
		push_usb_fifo_aaa(NORMAL_KB_DATA_TYPE, normal_key, 8);
		
	}
	if(last_consume_key!=consume_key)
	{
		last_consume_key=consume_key;
		push_usb_fifo_aaa(CONSUME_DATA_TYPE, (u8*)&consume_key, 2);
		
	}
	if(last_system_key!=system_key)
	{
		last_system_key=system_key;
		push_usb_fifo_aaa(SYSTEM_DATA_TYPE, &system_key, 1);
		
	}
		
	return 1;
}


void	callback_keyboard(u8 *p)
{
	static u8 seq_no_keyboard = 0;
	static u8 last_kb_data[sizeof(kb_data_t)]={0,0,0,0,0,0,0,0};
    rf_packet_keyboard_t *p_pkt = (rf_packet_keyboard_t *)(p + 8);

	u8 new_data_num = p_pkt->pno&0x0f;
    kb_tick=clock_time()|1;
    
	user_resume_host();
    if (p_pkt->seq_no != seq_no_keyboard)  	//skip same packet
    {
        seq_no_keyboard = p_pkt->seq_no;
        if ((reg_usb_host_conn==0)||(new_data_num==0))
		{
			return;
		}
         

		
#if (AES_METHOD ==0)
		u8 *p_dec=(u8*)&p_pkt->dat;
#elif((AES_METHOD ==2)||(AES_METHOD ==3))
		u8 *p_src=(u8*)&p_pkt->dat;
	set_aes_key(p_pkt->type);
	aes_user_ecb_encryption(&p_src[0],&dec_result[0]);
	if(new_data_num>1)
	{
		aes_user_ecb_encryption(&p_src[16],&dec_result[16]);
	}
	u8 *p_dec=(u8*)&dec_result[8];

	return;
	#if 0
		static u32 seq_no = 0;
		
		//if ((seq_no < dec.seq_no) || (seq_no == 0) || (0 == dec.seq_no))
		if ((seq_no != dec.seq_no))
		{
			seq_no = dec.seq_no;
		}
		else
		{
			return ;
		}
	#endif
#endif
   #if 0  
     u8 release_data[8] = {0,0,0,0,0,0,0,0};
      release_data[1]=p_dec[0];
	  release_data[2]=p_dec[1];
	  release_data[3]=p_dec[2];
     usb_mouse_hid_report_aaa(1,release_data,6);
	 return;
   #endif
	for(u8 i=0;i<new_data_num;i++)
	{
		//static u16 sn=0xffff;
		
		//if(sn!=p_dec[i*sizeof(mouse_data_t)+sizeof(mouse_data_t)-1])
		{
			//sn =p_dec[i*sizeof(mouse_data_t)+sizeof(mouse_data_t)-1];
			
			if(memcmp(last_kb_data,&p_dec[i*sizeof(kb_data_t)],sizeof(kb_data_t)))
			{
				memcpy(last_kb_data,&p_dec[i*sizeof(kb_data_t)],sizeof(kb_data_t));
				kb_data_handle(&p_dec[i*sizeof(kb_data_t)]);
			}
		}
	}


    }
}
void pull_rf_data()
{

	u8 i=0;
	if(rf_data_fifo.wptr!=rf_data_fifo.rptr)
	{
		RF_UI_DATA_S *p=(RF_UI_DATA_S*)rf_data_fifo.fifo[rf_data_fifo.rptr&(RF_DATA_FIFO_NUM-1)];
        u8 data_num=p->pno&0x0f;
        u8 *p_src=(u8*)&p->dat;
		u8 *p_dec;
		if(p->type==FRAME_TYPE_MOUSE)
		{
		#if (AES_METHOD ==0)
			p_dec=(u8*)&p->dat;
		#elif((AES_METHOD ==2)||(AES_METHOD ==3))
			p_dec=(u8*)&dec_result[8];
			set_aes_key(p->key_index);
			aes_user_ecb_encryption(&p_src[0],&dec_result[0]);
			if(data_num>1)
			{
				aes_user_ecb_encryption(&p_src[16],&dec_result[16]);
			}
		#endif
			mouse_data_t *now_ms_data;
			static u8 last_ms_btn;
			for(i=0;i<data_num;i++)
			{
				now_ms_data=(mouse_data_t *)&p_dec[i*sizeof(mouse_data_t)];
				//now_ms_data->btn=0;
				//now_ms_data->wheel=0;
				if(now_ms_data->x||now_ms_data->y||now_ms_data->wheel||(last_ms_btn!=now_ms_data->btn))
				{
					last_ms_btn=now_ms_data->btn;
					push_usb_fifo_aaa(MOUSE_DATA_TYPE,&p_dec[i*sizeof(mouse_data_t)],6);
				}
				
			}
		
		}
		else if(p->type==FRAME_TYPE_KEYBOARD)
		{
			static u8 last_kb_data[sizeof(kb_data_t)]={0,0,0,0,0,0,0,0};
		#if (AES_METHOD ==0)
			p_dec=(u8*)&p->dat;
		#elif((AES_METHOD ==2)||(AES_METHOD ==3))

			p_dec=(u8*)&dec_result[8];
			set_aes_key(p->key_index);
			aes_user_ecb_encryption(&p_src[0],&dec_result[0]);
			if(data_num>1)
			{
				aes_user_ecb_encryption(&p_src[16],&dec_result[16]);
			}
			
		#endif
			for(i=0;i<data_num;i++)
			{
				if(memcmp(last_kb_data,&p_dec[i*sizeof(kb_data_t)],sizeof(kb_data_t)))
				{
					memcpy(last_kb_data,&p_dec[i*sizeof(kb_data_t)],sizeof(kb_data_t));
					kb_data_handle(&p_dec[i*sizeof(kb_data_t)]);
				}
			}
	
		}
		rf_data_fifo.rptr++;
	}
}

u8 check_from_ram(rf_packet_pairing_t *p)
{
	static u16 buf[16]={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};

	if((memcmp(buf,&p->did,ID_LEN)))
	{
		memcpy(buf,&p->did,ID_LEN);
		return 0;
	}
	return 1;
}
u8 rx_id_same_cnt=0;

u8  pair_rx_response(u8 *p)
{
	static u32 rx_rssi = 0;
	static u8 rssi_last;
    rf_packet_pairing_t *p_pkt = (rf_packet_pairing_t *)(p + 8);
	u8 need_ack=0;
	last_device_type=0xff;
	if((p_pkt->flow & PKT_FLOW_PARING)&&(rf_pair_enable))
	{
		if (!rx_rssi)
		{
			rx_rssi = RECV_PKT_RSSI(p);
		}
		u8 rssi_cur = RECV_PKT_RSSI(p);
		if (abs(rssi_last - rssi_cur) < 17)
		{
			rx_rssi = ((rx_rssi << 1) + rssi_cur) / 3	;
		}
		rssi_last = rssi_cur;			
		int  rssi_paring_good = rx_rssi > (23 - custom_rssi_paring_th_offset); 
		if ( (rssi_paring_good))
		{
		
			if (check_from_ram(p_pkt)) //check ram
			{
				rx_id_same_cnt++;
			}
			else
			{
				rx_id_same_cnt=0;
			}
			
			if (rx_id_same_cnt>2) //check ram
			{
				rf_packet_ack_pairing_t *ack_pairing= (rf_packet_ack_pairing_t*)&rf_ack_buf;
				set_ack_buf_head(sizeof(rf_packet_ack_pairing_t),FRAME_TYPE_ACK);
				ack_pairing->gid1 = dongle_id;
				//ack_pairing.did = p_pkt->did;
				memcpy(&ack_pairing->did,&p_pkt->did,ID_LEN);
				
				p_post = (void *) set_device_id_in_firmware;
				//rx_id_same_cnt=5;
				need_ack=1; //send ACK
			}  
		}
		
	}
	//timer_cnt=0;
	//timer_tick=clock_time();
	return need_ack;
	
}

#if 1
_attribute_ram_code_ 
#endif
	u8   *rf_rx_response(u8 *p)
{
	//static u32 rx_rssi = 0;
	//static u8 rssi_last;
    rf_packet_pairing_t *p_pkt = (rf_packet_pairing_t *)(p + 8);
	
	u8 need_ack=0;
  //  static u32 binding_device[2] = {0, 0};
    if (p_pkt->proto == RF_PROTO_BYTE)
    {
        ///////////////  Paring/Link request //////////////////////////
        if ((rf_get_pipe(p) == PIPE_PARING))  	//paring/link request
        {
			#if 0
			last_device_type=0xff;
			if((p_pkt->flow & PKT_FLOW_PARING)&&(rf_pair_enable))
			{
				if (!rx_rssi)
		        {
		            rx_rssi = RECV_PKT_RSSI(p);
		        }
		        u8 rssi_cur = RECV_PKT_RSSI(p);
		        if (abs(rssi_last - rssi_cur) < 17)
		        {
		            rx_rssi = ((rx_rssi << 1) + rssi_cur) / 3   ;
		        }
		        rssi_last = rssi_cur;			
				int  rssi_paring_good = rx_rssi > (23 - custom_rssi_paring_th_offset); 
				if ( (rssi_paring_good))
			    {
				
				    if (check_from_ram(p_pkt)) //check ram
				    {
						rx_id_same_cnt++;
				    }
					else
					{
						rx_id_same_cnt=0;
					}
					
					if (rx_id_same_cnt>2) //check ram
					{
						rf_packet_ack_pairing_t *ack_pairing= (rf_packet_ack_pairing_t*)&rf_ack_buf;
						set_ack_buf_head(sizeof(rf_packet_ack_pairing_t),FRAME_TYPE_ACK);
						ack_pairing->gid1 = dongle_id;
		                //ack_pairing.did = p_pkt->did;
						memcpy(&ack_pairing->did,&p_pkt->did,ID_LEN);
						
						p_post = (void *) set_device_id_in_firmware;
						//rx_id_same_cnt=5;
						need_ack=1;	//send ACK
		            }  
			    }
				
			}
			//timer_cnt=0;
			//timer_tick=clock_time();
			#else
			 need_ack=pair_rx_response(p);
			#endif
            
        }
        else if (((rf_get_pipe(p) == PIPE_MOUSE)&&(p_pkt->type == FRAME_TYPE_MOUSE))
                 || ((rf_get_pipe(p) == PIPE_KEYBOARD)&&(p_pkt->type == FRAME_TYPE_KEYBOARD)))
        {
			rf_packet_mouse_t *p_km = (rf_packet_mouse_t *)(p + 8);
			#if DEBUG_GPIO_AAA	
			rx_data_gpio_toggle_debug();
			#endif
            u8 data_valid = 0;
			u8 key_index=0;
		#if 0	
            if (enc_key[p_pkt->type - FRAME_TYPE_MOUSE].did== p_pkt->did)
            {
                data_valid = 1;
				key_index= p_pkt->type - FRAME_TYPE_MOUSE;
            }
         #else  //for one device has mouse and kb data  
		     for(u8 i=0;i<2;i++)
		     {
			 	 if (enc_key[i].did== p_km->did)
			 	 {
				 	data_valid = 1;
					key_index= i;
					need_ack=1;	//send ACK
					break;
			 	 }
			 	
		     }
		 #endif
            if (data_valid)
            {	
            	u8 data_num = p_km->pno&0x0f;
                if (p_km->type == FRAME_TYPE_MOUSE)  //mouse
                {
					set_ack_buf_head(sizeof(rf_packet_ack_mouse_t),FRAME_TYPE_ACK_MOUSE);
                    frq_hopping_data.device_pktRcv_flg |= PKT_RCVD_FLG_MOUSE;
                    frq_hopping_data.mouse_pktRcv_sysTick = clock_time();
					 ms_tick=clock_time()|1;
                    //p_post = callback_mouse;
					static u8 seq_no_mouse = 0;
					if (p_km->seq_no != seq_no_mouse)
				    {
				        seq_no_mouse = p_km->seq_no;
						if ((reg_usb_host_conn)&&(data_num!=0))
						{
							push_rf_data_fifo(p_km->type,p_km->pno,key_index,(u8*)&p_km->dat,32);
						}
					}
                    
                }
                else   //keyboard
                {
					rf_packet_ack_keyboard_t *ack_keyboard= (rf_packet_ack_keyboard_t*)&rf_ack_buf;
      				ack_keyboard->status = host_keyboard_status;
					set_ack_buf_head(sizeof(rf_packet_ack_keyboard_t),FRAME_TYPE_ACK_KEYBOARD);
                    frq_hopping_data.device_pktRcv_flg |= PKT_RCVD_FLG_KB;
                    kb_tick=clock_time()|1;
					//p_post = callback_keyboard;

					static u8 seq_no_kb = 0;
					if (p_km->seq_no != seq_no_kb)
				    {
				        seq_no_kb = p_km->seq_no;
						if ((reg_usb_host_conn)&&(data_num!=0))
						{
							push_rf_data_fifo(p_km->type,p_km->pno,key_index,(u8*)&p_km->dat,32);
						}
					}
                   
                }
				user_resume_host();

				//timer_cnt=0;
				//timer_tick=clock_time();
				check_first_normal_data();
				
            }
			
        }
		//set_ack_buf_head(sizeof(rf_ack_empty_t),FRAME_NO_PAIR_FLOW);
    	//return (u8 *) &ack_empty->dma_len;
    	//timer_cnt=0;
		//timer_tick=clock_time();
    }
	if(need_ack)
	{
		return (u8*)&rf_ack_buf.dma_len;
	}
	else 
	{
		if(rf_pair_enable)
		{
			set_ack_buf_head(sizeof(rf_ack_empty_t),FRAME_TYPE_DEBUG);
			return (u8*)&rf_ack_buf.dma_len;
		}
		else
		{
			return NULL;
		}
		
	}
}

void  user_init(void)
{
    custom_init();

    ll_host_init((u8 *) &rf_ack_buf.dma_len);
	set_pair_flag();

    usb_dp_pullup_en(1);
    reg_wakeup_en = FLD_WAKEUP_SRC_USB;
    rf_set_power_level_index(RF_POWER_8dBm);
	analog_write (0xa3, analog_read(0xa3)&0xef);
	analog_write (0xa3, analog_read(0xa3)|0x10);
    //	reg_mcu_wakeup_mask = FLD_IRQ_TMR1_EN | FLD_IRQ_ZB_RT_EN | FLD_IRQ_USB_250US_EN;

	rf_set_channel(0, RF_CHN_TABLE);
	rf_set_rxmode();

}

void pull_usb_data()
{
#if 0
    if(rf_tx_flag)
    {
		return;
    }
#endif	
	int success=1;
	if(usb_fifo_aaa.wptr!=usb_fifo_aaa.rptr)
	{
		
		USB_DATA_S *p=(USB_DATA_S*)usb_fifo_aaa.fifo[usb_fifo_aaa.rptr&(USB_FIFO_NUM-1)];

		if(p->type==MOUSE_DATA_TYPE)
		{
			
			//if(memcmp_aaa(mouse_last_data, p->buf,6)||(p->buf[1]!=0)||(p->buf[2]!=0)||(p->buf[3]!=0)||(p->buf[4]!=0)||(p->buf[5]!=0))
			{
				//memcpy(mouse_last_data, p->buf,6);
				success=usb_mouse_hid_report_aaa(1,p->buf, 6);
			}
     		if((p->buf[0]==0)&&(p->buf[5]==0))
			{
				mouse_not_release=0;
			}
			else
			{
				mouse_not_release=1;
			}
			
		}
		else if(p->type==NORMAL_KB_DATA_TYPE)
		{
			//if(memcmp_aaa(normal_last_data, p->buf,8))
			{
				//memcpy(normal_last_data, p->buf,8);
				success=usb_keyboard_hid_report_aaa(p->buf);
			}
			
			
		}
		else if(p->type==CONSUME_DATA_TYPE)
		{
			
			//if(memcmp_aaa(consume_last_data, p->buf,2))
			{
				//memcpy(consume_last_data, p->buf,2);
				success=usb_mouse_hid_report_aaa(2,p->buf, 2);
			}
			
		}
		else if(p->type==SYSTEM_DATA_TYPE)
		{
			//if(sys_last_data!=p->buf[0])
			{
				//sys_last_data=p->buf[0];
				success=usb_mouse_hid_report_aaa(3,p->buf, 1);
			}
			
			
		}
		if(success)
		{
			usb_fifo_aaa.rptr++;
		}
	}
    else
    {
		u8 release_data[8] = {0,0,0,0,0,0,0,0};
		if(mouse_not_release&&ms_tick&&(clock_time_exceed(ms_tick,2000000)))
		{
			if(usb_mouse_hid_report_aaa(1,release_data, 6))
			{
				mouse_not_release=0;
				ms_tick=0;
			}
		}
		else if(has_consume_key_press&&kb_tick&&clock_time_exceed(kb_tick,2000000))
		{
			if(usb_mouse_hid_report_aaa(2,release_data, 2))
			{
				has_consume_key_press=0;
				kb_tick=0;
			}
		}
		else if(has_system_key_press&&kb_tick&&clock_time_exceed(kb_tick,2000000))
		{
			if(usb_mouse_hid_report_aaa(3,release_data, 1))
			{
				has_system_key_press=0;
				kb_tick=0;
			}
		}
		
		if(has_normal_key_press&&kb_tick&&clock_time_exceed(kb_tick,2000000))
		{
			if(usb_keyboard_hid_report_aaa(release_data))
			{
				has_normal_key_press=0;
				kb_tick=0;
			}
		}
		
    }
	
}




void main_loop(void)
{
    if (p_post)
    {
        (*p_post)(raw_pkt);
        p_post = NULL;
    }
    usb_handle_irq();

    usb_host_cmd_proc((u8 *)&rf_ack_buf.dma_len);

    //usb_data_report_proc();
    pull_rf_data();
    pull_usb_data();
#if (GOLD_TEST_DONGLE==0)

    paring_time_check();  //if exceed paring_time enable,  set rf_paring_enable to 0
#endif
    proc_suspend();
}
