/*
 * dongle_usb.c
 *
 *  Created on: Feb 13, 2014
 *      Author: xuzhen
 */

#include "../../proj/tl_common.h"
#include "../../proj_lib/rf_drv.h"
#include "../../proj_lib/pm.h"
#include "../common/rf_frame.h"
#include "dongle_usb.h"
#include "dongle_custom.h"
#include "../../proj/app/usbmouse_i.h"

#if(USB_DESCRIPTER_CONFIGURATION_FOR_KM_DONGLE)  //for km dongle customization (add by sihui)


static const u8 mouse_report_desc[] =
{
    /*******************mouse report=66************************/
    0x05, 0x01,  // Usage Page (Generic Desktop)
    0x09, 0x02,  // Usage (Mouse)
    0xA1, 0x01,  // Collection (Application)
    0x85, 0X01,  // Report Id
    0x09, 0x01,  //   Usage (Pointer)
    0xA1, 0x00,  //   Collection (Physical)
    0x05, 0x09,  //	 Usage Page (Buttons)
    0x19, 0x01,  //	 Usage Minimum (01) - Button 1
    0x29, 0x03,  //	 Usage Maximum (03) - Button 3
    0x15, 0x00,  //	 Logical Minimum (0)
    0x25, 0x01,  //	 Logical Maximum (1)
    0x75, 0x01,  //	 Report Size (1)
    0x95, 0x05,  //	 Report Count (3)
    0x81, 0x02,  //	 Input (Data, Variable, Absolute) - Button states
    0x75, 0x03,  //	 Report Size (5)
    0x95, 0x01,  //	 Report Count (1)
    0x81, 0x01,  //	 Input (Constant) - Padding or Reserved bits

    0x05, 0x01, 	 //  Usage Page (Generic Desktop Control)
    0x09, 0x30,  // Usage (X)
    0x09, 0x31,  // Usage (Y)

    0x16, 0x01, 0x80, //  LOGICAL_MINIMUM(0)
    0x26, 0xff, 0x7f,
    0x75, 0x10, //  Report Size (16)
    0x95, 0x02, //  Report Count (2)
    0x81, 0x06, //  Input (Data, Variable, Relative)

    //0x05,0x01,			 //  Usage Page (Generic Desktop Control)
    0x09, 0x38, 		 //  Usage (Wheel)
    0x15, 0x81, 		 //  Logical Minimum (-4)
    0x25, 0x7F, 		 //  Logical Maximum (3)
    0x75, 0x08, 		 //  Report Size (3)
    0x95, 0x01, 		 //  Report Count (1)
    0x81, 0x06, 		 //  Input (Data, Variable, Relative)

    0xC0,		  //   End Collection
    0xC0,		  // End Collection

    /*******************consume report=25************************/
    0x05, 0x0C,   // Usage Page (Consumer)
    0x09, 0x01,   // Usage (Consumer Control)
    0xA1, 0x01,   // Collection (Application)
    0x85, 0x02,   //	 Report Id
    0x75, 0x10, 	//global, report size 16 bits
    0x95, 0x01, 	//global, report count 1
    0x15, 0x01, 	//global, min  0x01
    0x26, 0x8c, 0x02, //global, max  0x28c
    0x19, 0x01, 	//local, min   0x01
    0x2a, 0x8c, 0x02, //local, max	  0x28c
    0x81, 0x00, 	//main,  input data varible, absolute
    0xc0,		  //main, end collection

    /*******************system power report=33************************/
    0x05, 0x01,     //gobal,  USAGE_PAGE 1 (Generic Desktop)
    0x09, 0x80,     //local, usage ID 0x80 system control
    0xa1, 0x01,     //main conllection
    0x85, 0x03,     //global report ID 0x4

    0x09, 0x82,     //local usage ID 0x82 system sleep
    0x09, 0x81,     //local usage ID 0x81 system power down
    0x09, 0x83,     //local usage ID 0x83 system wakeup

    0x15, 0x00,     //global min 01
    0x25, 0x01,     //gobal, max 3
    0x19, 0x01,  //	 Usage Minimum (01) - Button 1
    0x29, 0x03,  //	 Usage Maximum (03) - Button 3

    0x75, 0x01,    //global, report size 2
    0x95, 0x03,    //report count  1
    0x81, 0x02,    //main, input data, var, abs, No Prefer, NULL state
    //0x75,0x03,  //	 Report Size (5)
    0x95, 0x05, //	 Report Count (1)
    0x81, 0x01, //	 Input (Constant) - Padding or Reserved bits

    0xc0,        //end of collection

    /*******************user define  report=24************************/
    0x05, 0x01,    //global,  USAGE_PAGE 1 (Generic Desktop)
    0x09, 0x00,    //usage undefined
    0xa1, 0x01,    //main collection
    0x85, 0x05,    //global report ID 0x5
    0x06, 0x00, 0xff, //global usage page
    0x09, 0x01,    //local,  usage ID 01  Consumer Control
    0x15, 0x81,    //global min 81
    0x25, 0x7f,    //global, max 7f
    0x75, 0x08,    //global, report size 8
    0x95, 0x07,    //report count  7
    0xb1, 0x02,    //feature (data, var, abs)
    0xc0,         //main, end collection
};
static const u8 kb_report_desc[] =
{
    //keyboard report in
    0x05, 0x01,	 // Usage Pg (Generic Desktop)
    0x09, 0x06,	 // Usage (Keyboard)
    0xA1, 0x01,	 // Collection: (Application)
    //0x85, REPORT_ID_KEYBOARD_INPUT_AAA,	 // Report Id (keyboard)
    //
    0x05, 0x07,	 // Usage Pg (Key Codes)
    0x19, 0xE0,	 // Usage Min (224)  VK_CTRL:0xe0
    0x29, 0xE7,	 // Usage Max (231)  VK_RWIN:0xe7
    0x15, 0x00,	 // Log Min (0)
    0x25, 0x01,	 // Log Max (1)
    //
    // Modifier byte
    0x75, 0x01,	 // Report Size (1)   1 bit * 8
    0x95, 0x08,	 // Report Count (8)
    0x81, 0x02,	 // Input: (Data, Variable, Absolute)
    //
    // Reserved byte
    0x95, 0x01,	 // Report Count (1)
    0x75, 0x08,	 // Report Size (8)
    0x81, 0x01,	 // Input: (Constant)

    //keyboard output
    //5 bit led ctrl: NumLock CapsLock ScrollLock Compose kana
    0x95, 0x05,	//Report Count (5)
    0x75, 0x01,	//Report Size (1)
    0x05, 0x08,	//Usage Pg (LEDs )
    0x19, 0x01,	//Usage Min
    0x29, 0x05,	//Usage Max
    0x91, 0x02,	//Output (Data, Variable, Absolute)
    //3 bit reserved
    0x95, 0x01,	//Report Count (1)
    0x75, 0x03,	//Report Size (3)
    0x91, 0x01,	//Output (Constant)

    // Key arrays (6 bytes)
    0x95, 0x06,	 // Report Count (6)
    0x75, 0x08,	 // Report Size (8)
    0x15, 0x00,	 // Log Min (0)
    0x26, 0xF1, 0x00,	 // Log Max (241)
    0x05, 0x07,	 // Usage Pg (Key Codes)
    0x19, 0x00,	 // Usage Min (0)
    0x2a, 0xf1, 0x00,	 // Usage Max (241)
    0x81, 0x00,	 // Input: (Data, Array)

    0xC0,			  // End Collection
};

u8 *usbmouse_get_report_desc(void)
{
    return (u8 *)(mouse_report_desc);
}

u16 usbmouse_get_report_desc_size(void)
{
    return sizeof(mouse_report_desc);
}
u8 *usbkb_get_report_desc(void)
{
    return (u8 *)(kb_report_desc);
}

u16 usbkb_get_report_desc_size(void)
{
    return sizeof(kb_report_desc);
}

u16 len_total_km_cfg_desc = 0;

u8 device_desc_km[18] =
{
    0x12,  		// sizeof(USB_Descriptor_Device_t)
    0x01,  		// Type = DTYPE_Device
    0x10, 0x01, 	// 0x0110 : USBSpecification, USB 1.1
    0x00,       // Class : USB_CSCP_NoDeviceClass
    0x00, 		// SubClass : USB_CSCP_NoDeviceSubclass
    0x00, 		// Protocol : USB_CSCP_NoDeviceProtocol
    8,          // Endpoint0Size, Maximum Packet Size for Zero Endpoint. Valid Sizes are 8, 16, 32, 64
    0x8A, 0x24, // VendorID 閾囧浄鐗傜涵锟�0x248a   8  9
    (ID_PRODUCT & 0xff), (ID_PRODUCT >> 8) & 0xff, // ProductID defult value   mouse only:2781  mouse_keyboard_kit:2782
    0x00, 0x01, 	// 0x0100 : ReleaseNumber
    1, 			// ManufacturerStrIndex = USB_STRING_VENDOR   @@@ pay attention
    2,			// ProductStrIndex = USB_STRING_PRODUCT		  @@@ pay attention
    0, 			// SerialNumStrIndex
    1           // NumberOfConfigurations
};
u8 configuration_km_desc[9 + 9 + 9 + 7 + 9 + 9 + 7] =
{
    //USB_Descriptor_Configuration_Header_t
    0x09,			//sizeof(USB_Descriptor_Configuration_Header_t)
    0x02,  			//type= DTYPE_Configuration
    0x09, 0x00,     //TotalConfigurationSize,  @@@@ will change @@@@
    0x01,           //TotalInterfaces          @@@@ will change @@@@
    1,      		//Configuration index
    0,      		//ConfigurationStrIndex = NO_DESCRIPTOR
    0xa0,           //ConfigAttributes = USB_CONFIG_ATTR_RESERVED | USB_CONFIG_ATTR_REMOTEWAKEUP
    0x19,           	//MaxPowerConsumption = USB_CONFIG_POWER_MA(50)  MaxPower = 100mA

    //USB_Descriptor_Interface_t mouse_interface
    0x09, 			 //size = sizeof(USB_Descriptor_Interface_t)
    0x04, 	  		 //type = DTYPE_Interface
    0,               //InterfaceNumber          @@@@ will change @@@@
    0,     			 //AlternateSetting = 0
    1,  			 //TotalEndpoints = 1
    0x03,			 //Class = HID_CSCP_HIDClass
    0x01,        	 //SubClass = HID_CSCP_BootSubclass,
    0x02,			 //Protocol = HID_CSCP_MouseBootProtocol
    0,            	 //InterfaceStrIndex = NO_DESCRIPTOR
    //USB_HID_Descriptor_HID_t   mouse_hid
    0x09, 			 //size = sizeof(USB_HID_Descriptor_HID_t)
    0x21,       	 //type = HID_DTYPE_HID
    0x11, 0x01,      //HIDSpec = 0x0111
    0x21, 			 //CountryCode = USB_HID_COUNTRY_US
    1,               //TotalReportDescriptors
    0x22,            //HIDReportType = HID_DTYPE_Report
    sizeof(mouse_report_desc), 0x00,      //HIDReportLength[2] = {sizeof(mouse_report_desc), 0x00}
    //USB_Descriptor_Endpoint_t   mouse_in_endpoint
    0x07, 			//size = sizeof(USB_Descriptor_Endpoint_t)
    0x05,     		//type = DTYPE_Endpoint
    ENDPOINT_DIR_IN | USB_EDP_MOUSE, //EndpointAddress = ENDPOINT_DIR_IN | USB_EDP_MOUSE,
    0x03,   		//Attributes = EP_TYPE_INTERRUPT
    0x08, 0x00, 	 	//EndpointSize = 0x0008
    2,    			//PollingIntervalMS = USB_MOUSE_POLL_INTERVAL


    //USB_Descriptor_Interface_t keyboardInterface
    0x09, 			 //size = sizeof(USB_Descriptor_Interface_t)
    0x04, 	 		 //type = DTYPE_Interface
    1,               //InterfaceNumber		    @@@@ will change @@@@
    0,     			 //AlternateSetting = 0
    1,  			 //TotalEndpoints = 1
    0x03,			 //Class = HID_CSCP_HIDClass
    0x01,            //SubClass = HID_CSCP_BootSubclass,
    0x01,            //Protocol = HID_CSCP_KeyboardBootProtocol,
    0,           	 //InterfaceStrIndex = NO_DESCRIPTOR
    //USB_HID_Descriptor_HID_t   keyboard_hid
    0x09,     		 //size = sizeof(USB_HID_Descriptor_HID_t)
    0x21,            //type = HID_DTYPE_HID
    0x11, 0x01, 	 // HIDSpec = 0x0111
    0x21, 			 //CountryCode = USB_HID_COUNTRY_US
    1, 		  		 //TotalReportDescriptors
    0x22, 			 //HIDReportType = HID_DTYPE_Report
    sizeof(kb_report_desc), 0x00,      //HIDReportLength[2] = {sizeof(keyboard_report_desc), 0x00}
    //USB_Descriptor_Endpoint_t  keyboard_in_endpoint
    0x07, 			 //size = sizeof(USB_Descriptor_Endpoint_t)
    0x05, 	    	 //type = DTYPE_Endpoint
    ENDPOINT_DIR_IN | USB_EDP_KEYBOARD_IN, //EndpointAddress = ENDPOINT_DIR_IN | USB_EDP_KEYBOARD_IN,
    0x03,    		 //Attributes = EP_TYPE_INTERRUPT
    0x08, 0x00,		 //EndpointSize = 0x0008
    0x0a,   		     //PollingIntervalMS = USB_KEYBOARD_POLL_INTERVAL = 10

};





void get_usb_descripter_configuration(void)
{
    if (p_custom_cfg->vendor_id != U16_MAX)
    {
        device_desc_km[8] =  p_custom_cfg->vendor_id & 0xff;
        device_desc_km[9] = (p_custom_cfg->vendor_id > 8) & 0xff;
    }
    if (p_custom_cfg->prodct_id != U16_MAX)
    {
        device_desc_km[10] =  p_custom_cfg->prodct_id & 0xff;
        device_desc_km[11] = (p_custom_cfg->prodct_id >> 8) & 0xff;
    }

    if (dongle_support_keyboard_enable)
    {
        //device_desc_km[10] +=1;//set pid
        len_total_km_cfg_desc = 9 + 9 + 9 + 7 + 9 + 9 + 7;
        configuration_km_desc[4] = 2; //interface  num config
    }
    else
    {
        len_total_km_cfg_desc = 9 + 9 + 9 + 7;
        configuration_km_desc[4] = 1; //interface  num
    }
    configuration_km_desc[2] = len_total_km_cfg_desc;

   // if (p_custom_cfg->report_type == 0) //mouse report  250
   // {
       // configuration_km_desc[33] = 2; //PollingIntervalMS = USB_MOUSE_POLL_INTERVAL
   // }

    set_string_from_addr(&vendor_desc_km, VENDOR_STRING_ADDR);
    set_string_from_addr(&prodct_desc_km, PRODCT_STRING_ADDR);
    set_string_from_addr(&serial_desc_km, SERIAL_STRING_ADDR);


}


#endif






KM_USB_Descriptor_String_t  vendor_desc_km =
{
    14,               //sizeof(L"Telink")=14
    0x03,			  //DTYPE_String=0x03
    L"Telink"
};

KM_USB_Descriptor_String_t  prodct_desc_km =
{
    36,               //sizeof(L"Wireless Receiver")=36
    0x03,			  //DTYPE_String=0x03
    L"Wireless Receiver"
};

KM_USB_Descriptor_String_t  serial_desc_km =
{
    18,               //sizeof(L"TLSR8566")=18
    0x03,			  //DTYPE_String=0x03
    L"TLSR8366"
};

void set_string_from_addr(KM_USB_Descriptor_String_t *ptr, u16 addr)
{
    u8 *p;

#if(CUSTOM_DATA_ERR_FIX_ENABLE)
    for (int i = 0; i < CUSTOM_DATA_MAX_COUNT; i++)
    {
        p = (u8 *)(addr + (i << 5));
        if (*p != 0)
        {
            break;
        }
    }
#else
    p = (u8 *)(addr);
#endif

    int j = 0;
    if (*p != 0xff)
    {
        while (*p != 0xff)
        {
            ptr->str[j++] = *p++;
        }
        ptr->str[j] = 0;
        ptr->Size = (j + 1) << 1;
    }

    /*
    u8 len;
    u8 *p = (u8 *)(addr);
    if(*p != U8_MAX){
    	len = (*p++)%MAX_STRING_LEN;
    	ptr->Size = (len+1)<<1;
    	for(int i=0;i<len;i++){
    		ptr->str[i] = *p++;
    	}
    	ptr->str[len] = 0;
    }
    */
}


extern u8 rf_tx_flag;
extern void usbmouse_report_frame(void);
extern void usbmouse_release_check(void);
extern void usbkb_report_frame(void);
extern void usbkb_release_check(void);

void usb_data_report_proc(void)
{
    if ((reg_irq_src & FLD_IRQ_USB_PWDN_EN))
    {
        return;
    }


    if ((rf_tx_flag == 0))
    {
       // usbmouse_report_frame();
       // usbmouse_release_check();
    }

#if USB_KEYBOARD_ENABLE

    if (dongle_support_keyboard_enable && (rf_tx_flag == 0))
    {
        //usbkb_report_frame();
        //usbkb_release_check();
    }
#endif
}

