/*
 * rf_ll.c: link layer
 *
 *  Created on: Mar 5, 2014
 *      Author: MZ
 */
#include "../../proj/tl_common.h"
#include "../../proj/mcu/watchdog_i.h"
#include "../../proj_lib/rf_drv.h"
#include "../../proj_lib/pm.h"
#include "../common/rf_frame.h"
#include "../link_layer/rf_ll.h"
void rf_send_single_packet(void *addr);
//extern u8 *p_debug_pkt;
extern rf_packet_ack_t rf_ack_buf;

//u8 timer_cnt=0;
//u32 timer_tick=0;
u8		chn_mask = 0x80;

#define	CHANNEL_HOPPING_ALWAYS				1

#define			CHANNEL_SLOT_TIME			8000

#define		PKT_BUFF_SIZE		96
#define     NUM_PKT_FIFO        2

#define		FH_CHANNEL_PKT_RCVD_MAX			8
#define		LL_CHANNEL_SYNC_TH				2
#define		LL_CHANNEL_SEARCH_TH			60
#define		LL_CHANNEL_SEARCH_FLAG			BIT(16)

#define		LL_NEXT_CHANNEL(c)				((c + 6) & 14)

frqHopping_data_t  frq_hopping_data =
{
    FRQ_HOPPING_MODE_NORMAL,
    8,
    2,
    0,
    120000,
    0
};

unsigned char  	rf_rx_buff[PKT_BUFF_SIZE * NUM_PKT_FIFO] __attribute__((aligned(4)));
int		rf_rx_wptr;
int  rf_rx_rptr=0;
u8		host_channel = 0;

u8		ll_chn_sel;
u8		ll_chn_pkt[16] = {0};
int		device_packet_received;
#if DEBUG_GPIO_AAA
void t8ms_gpio_toggle_debug()
{
	
	//static u8 flag=0;
	//flag^=0xff;
	//gpio_write(PIN_DEBUG_8MS,flag);
}
void change_freq_gpio_toggle_debug()
{
	//static u8 flag=0;
	//flag^=0xff;
	//gpio_write(PIN_DEBUG_CHANGE_FREQ,flag);
}

void rx_int_gpio_toggle_debug()
{
	//static u8 flag=0;
	//flag^=0xff;
	//gpio_write(PIN_DEBUG_RX_INT,flag);
}
void rx_crc_gpio_toggle_debug()
{
	//static u8 flag=0;
	//flag^=0xff;
	//gpio_write(PIN_DEBUG_RX_CRC,flag);
}
void rx_data_gpio_toggle_debug()
{
	//static u8 flag=0;
	//flag^=0xff;
	//gpio_write(PIN_DEBUG_RX_DATA,flag);
}

#endif


u8	get_next_channel_with_mask(u32 mask, u8 chn)
{
#if DEBUG_GPIO_AAA
		change_freq_gpio_toggle_debug();
#endif

    int chn_high = (mask >> 4) & 0x0f;

    if (mask & LL_CHANNEL_SEARCH_FLAG)
    {
        return LL_NEXT_CHANNEL(chn);
    }
    else if (chn_high != chn)
    {
        ll_chn_sel = 1;
        return chn_high;
    }
    else
    {
        ll_chn_sel = 0;
        return mask & 0x0f;
    }
}


u8	update_channel_mask(u32 mask, u8 chn, u8 *chn_pkt)
{
    static int ll_chn_sel_chg, ll_chn_hold;
    static u32	tick_fh = 0;
    //static u32	fh_num = 0;
#if CHANNEL_HOPPING_ALWAYS
    if (device_packet_received == 1)
    {
        tick_fh = clock_time();
    }
    else if (clock_time_exceed(tick_fh, frq_hopping_data.fre_hp_always_time_us))
    {
        chn_pkt[ll_chn_sel_chg] = 0;
        chn_pkt[!ll_chn_sel_chg] = frq_hopping_data.frq_hp_chn_pkt_rcvd_max;
    }
#endif

    if (ll_chn_hold)
    {
        ll_chn_hold--;
        chn_pkt[0] = chn_pkt[1] = 0;
    }
    int diff = chn_pkt[ll_chn_sel] - chn_pkt[!ll_chn_sel];
    int hit_th = diff > frq_hopping_data.frq_hp_hit_diff_num;
    if (chn_pkt[ll_chn_sel] >= frq_hopping_data.frq_hp_chn_pkt_rcvd_max || hit_th)
    {
        int dual_chn[2];
        dual_chn[0] = mask & 0x0f;
        dual_chn[1] = mask >> 4;
        if (hit_th)   //change channel
        {
            ll_chn_hold = 4;
            chn = dual_chn[!ll_chn_sel];
            for (int i = 0; i < 8; i++)
            {
                chn = LL_NEXT_CHANNEL(chn);
                if ((ll_chn_sel && chn != dual_chn[1]))
                {
                    mask = (mask & 0xf0) | chn;
                    break;
                }
                else if (!ll_chn_sel && chn != dual_chn[0])
                {
                    mask = (mask & 0x0f) | (chn << 4);
                    break;
                }
            }
            tick_fh = clock_time();
            //fh_num++;
            ll_chn_sel_chg = !ll_chn_sel;		//remember latest channel change
        }
        chn_pkt[0] = chn_pkt[1] = 0;
    }
    return mask;
}


//u8 	*p_debug_pkt = 0;
//static u32 tick_last_tx;
static u32 tick_timer1_irq;

void ll_host_init(u8 *pkt)
{
    rf_multi_receiving_init(BIT(0) | BIT(1) | BIT(2));
    //p_debug_pkt = pkt;
    reg_dma_rf_tx_addr = (u16)(u32) pkt;
    reg_dma_rf_rx_addr = (u16)(u32)(rf_rx_buff);
    reg_dma2_ctrl = FLD_DMA_WR_MEM | (PKT_BUFF_SIZE >> 4); // rf rx buffer enable & size
    reg_dma_chn_irq_msk = 0;

    reg_rf_irq_mask = FLD_RF_IRQ_RX | FLD_RF_IRQ_TX;


#if (FREQ_HOPPING_METHOD==1)

    reg_tmr1_tick = 0;
    reg_tmr1_capt = CLOCK_SYS_CLOCK_1US * CHANNEL_SLOT_TIME;
    reg_tmr_ctrl |= FLD_TMR1_EN;

    reg_irq_mask |= FLD_IRQ_TMR1_EN| FLD_IRQ_ZB_RT_EN;    //enable RF & timer1 interrupt

#else

	reg_irq_mask |=  FLD_IRQ_ZB_RT_EN;
#endif
 

}

//u8 rf_tx_flag = 0;
#if (FREQ_HOPPING_METHOD==1)

void irq_host_timer1(void)
{
	#if DEBUG_GPIO_AAA
	t8ms_gpio_toggle_debug();
#endif

    tick_timer1_irq++;
    if (device_packet_received == 1)
    {
        ll_chn_pkt[ll_chn_sel]++;
    }

    if (dongle_support_keyboard_enable)
    {
        frq_hopping_mode_switch();
    }

    host_channel = get_next_channel_with_mask(chn_mask, host_channel);

   
    chn_mask = update_channel_mask(chn_mask, host_channel, ll_chn_pkt);
    
    device_packet_received = 0;
#if 0
    set_ack_buf_head(sizeof(rf_packet_debug_t),FRAME_TYPE_DEBUG);
    if (p_debug_pkt && clock_time_exceed(tick_last_tx, 500000))
    {
        //rf_stop_trx ();
        rf_set_channel(15, RF_CHN_TABLE);
        //rf_set_tx_pipe (PIPE_MOUSE);
        //RF_TX_PA_POWER_LEVEL (0);
        ((rf_packet_ack_pairing_t *)p_debug_pkt)->tick = reg_tmr1_tick;
        ((rf_packet_ack_pairing_t *)p_debug_pkt)->chn = chn_mask;
        ((rf_packet_ack_pairing_t *)p_debug_pkt)->info1++;
        //rf_multi_receiving_send_packet (p_debug_pkt);
        rf_set_power_level_index(RF_POWER_LEVEL_MIN);
        rf_send_single_packet(p_debug_pkt);
        //TxPkt (p_debug_pkt);
        tick_last_tx = clock_time();
    }
    else
	#endif
    {
		//if(rf_tx_flag==0)
		{
        	rf_set_channel(host_channel, RF_CHN_TABLE);
        	rf_set_rxmode();
		}
    }
}
#endif
u8 *raw_pkt = NULL;
callback_rx_func p_post = NULL;
#if 1
_attribute_ram_code_ 
#endif
void  user_rf_start_tx()
{
	u8 *p_ack=(u8*)&rf_ack_buf.dma_len;
	//rf_set_power_level_index(RF_POWER_LEVEL_MAX);

	//rf_set_ack_packet(p_ack);
#if DEBUG_GPIO_AAA
	gpio_write(PIN_DEBUG_TX_DATA,1);
#endif	
	//rf_tx_flag = 1;
	//rf_start_stx(&rf_ack_buf, t+30*CLOCK_SYS_CLOCK_1US);	
	rf_start_stx(&rf_ack_buf, clock_time()+25*CLOCK_SYS_CLOCK_1US);

	// assume running @ 32MHz, 32M/1024: 32 us resolution
	((rf_packet_ack_pairing_t *)p_ack)->rssi = raw_pkt[4];
	((rf_packet_ack_pairing_t *)p_ack)->chn = chn_mask;
	((rf_packet_ack_pairing_t *)p_ack)->per =
	(ll_chn_pkt[0] & 0xf) | ((ll_chn_pkt[1] & 0xf) << 4) ;
	//((rf_packet_ack_mouse_t *)p_ack)->info = fh_num;
#if(CLOCK_SYS_CLOCK_HZ == 32000000)
	((rf_packet_ack_pairing_t *)p_ack)->tick = ((reg_tmr1_tick >> 10) & 0xff) |
	((tick_timer1_irq & 0xff) << 8);
#elif(CLOCK_SYS_CLOCK_HZ == 24000000)
	((rf_packet_ack_pairing_t *)p_ack)->tick = ((reg_tmr1_tick * 170 >> 17) & 0xff) |
	((tick_timer1_irq & 0xff) << 8);
#else				// 16 MHz
	((rf_packet_ack_pairing_t *)p_ack)->tick = ((reg_tmr1_tick >> 9) & 0xff) |
	((tick_timer1_irq & 0xff) << 8);
#endif

}


_attribute_ram_code_ void irq_host_rx(void)
{
  //  u32 t=clock_time();
    raw_pkt = (u8 *)(rf_rx_buff + rf_rx_wptr * PKT_BUFF_SIZE);
    rf_rx_wptr = (rf_rx_wptr + 1) %NUM_PKT_FIFO;
    reg_dma_rf_rx_addr = (u16)(u32)(rf_rx_buff + rf_rx_wptr * PKT_BUFF_SIZE);  //set next buffer

    reg_rf_irq_status = FLD_RF_IRQ_RX;
#if DEBUG_GPIO_AAA	
	rx_int_gpio_toggle_debug();
 #endif  
    //rf_packe_header_t *p = (rf_packet_header_t*)(((u8*)raw_pkt) + 8);
    if	(raw_pkt[0] >= 15 &&
            RF_PACKET_LENGTH_OK(raw_pkt) &&
            RF_PACKET_CRC_OK(raw_pkt))
    {
		#if DEBUG_GPIO_AAA	
        rx_crc_gpio_toggle_debug();
		#endif
        // scheduling TX, 150 us interval
        rf_set_tx_rx_off();
		
		#if 0
        //		u32 t = *((u32 *) (raw_pkt + 8));
        u32 t = *((u32 *)(0x800460));

     
           t += ((raw_pkt[12] + 8) * 4 + 50) * CLOCK_SYS_CLOCK_1US;
      

        u32 diff = t - clock_time();
        if (diff > (300 * CLOCK_SYS_CLOCK_1US))
        {
            t = clock_time() + 5 * CLOCK_SYS_CLOCK_1US;
        }
        rf_start_stx(&rf_ack_buf, t);
		#else
        
		#endif
		


        u8 *p_ack = rf_rx_response(raw_pkt);
		
        if (p_ack)
        {
            	
        }
        else  		//cancel TX
        {
			//rf_tx_flag=0;
           // rf_stop_trx ();
			 rf_set_channel(host_channel, RF_CHN_TABLE);
    		rf_set_rxmode();
    		
    		
        }
			
        raw_pkt[0] = 1;
        device_packet_received++;
    }
}
#if 1
_attribute_ram_code_
#endif
void irq_host_tx(void)
{
   // rf_tx_flag = 0;
  //  tick_last_tx = clock_time();

    rf_set_channel(host_channel, RF_CHN_TABLE);
    rf_set_rxmode();
    
	//tx_gpio_toggle_debug();
#if DEBUG_GPIO_AAA
				gpio_write(PIN_DEBUG_TX_DATA,0);
			#endif
	
    reg_rf_irq_status = FLD_RF_IRQ_TX;
}

