
#pragma once

#include "usbmouse.h"
#include "../drivers/usb.h"
#include "../drivers/usbhw.h"
#include "../drivers/usbhw_i.h"
#include "../../vendor/common/default_config.h"
//#include "../../proj_lib/ble/service/hids.h"

/** HID class report descriptor. This is a special descriptor constructed with values from the
 *  USBIF HID class specification to describe the reports and capabilities of the HID device. This
 *  descriptor is parsed by the host and its contents used to determine what data (and in what encoding)
 *  the device will send, and what it may be sent back from the host. Refer to the HID specification for
 *  more details on HID report descriptors.
 */
/* debug note: */
/*
 0x85, 0x01, //Report ID (1) keyboard
 0x85, 0x02, //report ID 02 mouse
 ǷҪ , 1,2Ƿȷ
 */
 
static const USB_Descriptor_HIDReport_Datatype_t mouse_report_desc[] = 
{
	
	
#if 1//mouse report in  81 bytes
		
		0x05, 0x01,  // Usage Page (Generic Desktop)
		0x09, 0x02,  // Usage (Mouse)
		0xA1, 0x01,  // Collection (Application)
		0x85, HID_REPORT_ID_MOUSE_INPUT,  // Report Id
		0x09, 0x01,  //   Usage (Pointer)
		0xA1, 0x00,  //   Collection (Physical)
		0x05, 0x09,  // 	Usage Page (Buttons)
		0x19, 0x01,  // 	Usage Minimum (01) - Button 1
		0x29, 0x03,  // 	Usage Maximum (03) - Button 3
		0x15, 0x00,  // 	Logical Minimum (0)
		0x25, 0x01,  // 	Logical Maximum (1)
		0x75, 0x01,  // 	Report Size (1)
		0x95, 0x05,  // 	Report Count (3)
		0x81, 0x02,  // 	Input (Data, Variable, Absolute) - Button states
		0x75, 0x03,  // 	Report Size (5)
		0x95, 0x01,  // 	Report Count (1)
		0x81, 0x01,  // 	Input (Constant) - Padding or Reserved bits
	
		0x05,0x01,		//	Usage Page (Generic Desktop Control)	
		0x09,0x30,	 // Usage (X)
		0x09,0x31,	 // Usage (Y)
	
		0x16,0x01,0x80, //	LOGICAL_MINIMUM(0)
		0x26,0xff,0x7f,  
		0x75,0x10,	//	Report Size (16)
		0x95,0x02,	//	Report Count (2)
		0x81,0x06,	//	Input (Data, Variable, Relative)
	
		
	//	0x05,0x01,			//	Usage Page (Generic Desktop Control)  //for win7
		0x09,0x38,			//	Usage (Wheel)
		0x15,0x81,			//	Logical Minimum (-4)
		0x25,0x7F,			//	Logical Maximum (3)
		0x75,0x08,			//	Report Size (3)
		0x95,0x01,			//	Report Count (1)
		0x81,0x06,			//	Input (Data, Variable, Relative)

        0x05,0x0c,			// Usage Page(Consumer)
		0x0a,0x38,0x02,		// Usage (AC Pan) tilt
		0x15,0x81,			//	Logical Minimum (-128)
		0x25,0x7F,			//	Logical Maximum (127)
		0x75,0x08,			//	Report Size (8)
		0x95,0x01,			//	Report Count (3)
		0x81,0x06,			// Input (Data, Variable, Relative)
	
		
		0xC0,		 //   End Collection
		0xC0,		 // End Collection
	
#endif

	
#if 1//CONSUMER_REPORT_BYTE_AAA  25bytes

	//consumer report in
	0x05, 0x0C,   // Usage Page (Consumer)
	0x09, 0x01,   // Usage (Consumer Control)
	0xA1, 0x01,   // Collection (Application)
	0x85, HID_REPORT_ID_CONSUME_CONTROL_INPUT,   //     Report Id
	0x75,0x10,     //global, report size 16 bits
	0x95,0x01,     //global, report count 1
	0x15,0x01,     //global, min  0x01
	0x26,0x8c,0x02,  //global, max  0x28c
	0x19,0x01,     //local, min   0x01
	0x2a,0x8c,0x02,  //local, max    0x28c
	0x81,0x00,     //main,  input data varible, absolute
	0xc0,        //main, end collection	

#endif

#if 1//SYSTEM_KEY_FUN_ENABLE_AAA  29 bytes

0x05,0x01, 		//Usage Page (Generic Desktop Control)
0x09,0x80,			//Usage (SYSTEM CONTROL)
0xA1,0x01,	//Collection (Application)

0x85,HID_REPORT_ID_SYSTEM_INPUT,		//Report ID (ACPI)

0x25,0x01,			//	Logical Maximum (1)
0x15,0x00,			//	Logical Minimum (0)
0x75,0x01,			//	Report Size


0x09,0x82, 	//	USAGE SYSTEM SLEEP
0x09,0x81,	//	USAGE SYSTEM POWER DOWN 
0x09,0x83, 	//	USAGE SYSTEM WAKE UP

0x95,0x03,			//	REPORT_COUNT (03H)
0x81,0x02,			//	INPUT (DATA, VAR)								 
0x95,0x05,			//	Report Count (05)

0x81,0x01,			//	Input (CONSTANT)
0xC0,				//	END COLLECTION

#endif


#if 1  //24
	0x05,0x01,    // USAGE_PAGE (Generic Desktop)
	0x09,0x00,	  //usage undefined
	0xa1,0x01,	  //main collection
	0x85,HID_REPORT_ID_USER_FEATURE,	  //global report ID 0x5
	0x06,0x00,0xff,	//global usage page
	0x09,0x01,	  //local,	usage ID 01  Consumer Control
	0x15,0x81,	  //global min 81
	0x25,0x7f,	  //global, max 7f
	0x75,0x08,	  //global, report size 8
	0x95,0x07,	  //report count  7
	0xb1,0x02,	  //feature (data, var, abs)
	0xc0, //main, end collection
#endif

#if 1 //24
	
		0x05,0x01,	  // USAGE_PAGE (Generic Desktop)
		0x09,0x00,	  //usage undefined
		0xa1,0x01,	  //main collection
		0x85,HID_REPORT_ID_SPP_INPUT,	  //global report ID 0x5
		0x06,0x00,0xff, //global usage page
		0x09,0x01,	  //local,	usage ID 01  Consumer Control
		0x15,0x81,	  //global min 81
		0x25,0x7f,	  //global, max 7f
		0x75,0x08,	  //global, report size 8
		0x95,0x07,	  //report count  7
		0x81,0x02,	  //feature (data, var, abs)
		0xc0, //main, end collection

#endif




};

static inline u8* usbmouse_get_report_desc(void) {
	return (u8*) (mouse_report_desc);
}

static inline u16 usbmouse_get_report_desc_size(void) {
	return sizeof(mouse_report_desc);
}

